"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isMemoizingIterable = exports.MemoizingIterable = void 0;
/* @internal */
class MemoizingIterable {
    constructor(iter) {
        if (iter instanceof Array) {
            this.evaluatedData = iter;
        }
        else {
            this.evaluatedData = [];
            this.partialState = {
                evaluationIndex: 0,
                evaluationIter: iter[Symbol.iterator](),
            };
        }
    }
    [Symbol.iterator]() {
        return new MemoizingIterator(this);
    }
    toArray() {
        this.realize();
        return this.evaluatedData;
    }
    realize() {
        if (this.partialState !== undefined) {
            let r = this.partialState.evaluationIter.next();
            while (!r.done) {
                this.evaluatedData.push(r.value);
                r = this.partialState.evaluationIter.next();
            }
            this.partialState = undefined;
        }
    }
}
exports.MemoizingIterable = MemoizingIterable;
/* @internal */
function isMemoizingIterable(it) {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    return "evaluatedData" in it && Array.isArray(it.evaluatedData);
}
exports.isMemoizingIterable = isMemoizingIterable;
class MemoizingIterator {
    constructor(mem) {
        this.mem = mem;
        this.index = 0;
    }
    next() {
        if (this.mem.partialState !== undefined) {
            const state = this.mem.partialState;
            if (this.index < state.evaluationIndex) {
                const value = this.mem.evaluatedData[this.index++];
                return { done: false, value };
            }
            else {
                const r = state.evaluationIter.next();
                if (r.done) {
                    this.mem.partialState = undefined;
                    return r;
                }
                else {
                    this.mem.evaluatedData.push(r.value);
                    this.index++;
                    state.evaluationIndex++;
                    return {
                        done: false,
                        value: r.value,
                    };
                }
            }
        }
        else {
            if (this.index < this.mem.evaluatedData.length) {
                return {
                    done: false,
                    value: this.mem.evaluatedData[this.index++],
                };
            }
            return { done: true, value: undefined };
        }
    }
}
